; SUB.inc =================================================================

; BCD value in W -> 7-segment display code to W
; Value will be placed to port B.
; 0=segment off, 1=segment on
tab1    addwf   pc, f
        retlw   b'11000000'     ; displays 0
        retlw   b'11111001'     ; displays 1
        retlw   b'10100100'     ; displays 2
        retlw   b'10110000'     ; displays 3
        retlw   b'10011001'     ; displays 4
        retlw   b'10010010'     ; displays 5
        retlw   b'10000010'     ; displays 6
        retlw   b'11111000'     ; displays 7
        retlw   b'10000000'     ; displays 8
        retlw   b'10010000'     ; displays 9
; .........................................................................
; Display anode encoding according to cnt_m, anodes are connected to port A.
; 0=display digit on, 1=display digit off
tab2    addwf   pc, f
        retlw   b'11110111'     ; cnt_m=0 - anode 3 active
        retlw   b'11111011'     ; cnt_m=1 - anode 2 active
        retlw   b'11111101'     ; cnt_m=2 - anode 1 active
        retlw   b'11111110'     ; cnt_m=3 - anode 0 active
; .........................................................................
; Initialization of ports and system status. Register hrs, min, sec clear.
Init    movlw   0xFF            ; port A setup
        movwf   porta           ; display drivers off
        clrf    portb           ; port B setup (clear)

        movlw   b'11100000'
        Bank1
        movwf   trisa           ; port A is output
        movlw   00h
        movwf   trisb           ; port B is output
        movlw   b'10000011'     ; tmr0 clock internal, prescaler 1:16
        movwf   optireg         ; pull-ups disabled (required!)
        Bank0
        clrf    stat1           ; initial system status setup
        clrf    cnt_m           ; multiplex counter clear
        clrf    hrs             ; clear hours
        clrf    min             ; clear minutes
        clrf    sec             ; clear seconds
        clrf    new             ; clear button status register - new value
        clrf    old             ; clear button status register - old value
        return                  ; setup is complete
; .........................................................................
; Update routine. Input is a register address which is to be updated (stored
; in fsr before the call) and the value for overflow (the maxim register).
; The routine increments (with BCD adjust) the register pointed by the fsr.
; If this incremented value equels maxim this register is cleared and Z is
; set to 1.
update  movlw   .7
        addwf   indf, f         ; increments ind with BCD adjust
        btfsc   status, dcarry
        goto    akt_a
        movlw   .6
        subwf   indf, f
akt_a   movf    maxim, w
        xorwf   indf, w
        btfsc   status, zero    ; indf = maxim? (Z=1?)
        clrf    indf            ; yes => clear indf
        retlw   00h             ; no  => maxim

;***************************************************************************
; The routine provides wait 5ms (not from the moment when th routine is called
; but from last tmr0 reading).
w5ms    comf    tmr0, w
        btfss   status, zero    ; tmr0 = 0ffh? (Z=1?)
        goto    w5ms            ; no  => wait
                                ; yes => test overflow
w5msa   btfsc   tmr0, 0         ; tmr0 overflew? (bit0=0?)
        goto    w5msa           ; no  => wait for interrupt
        retlw   0               ; yes => maxim
;***************************************************************************
; The routine sets one of four digits for 5ms. The digit is selected according
; the hm bit in the stat1 register and the value in the multiplex counter cnt_m.
disply  movlw   0ffh
        movwf   porta           ; disconnect display anodes
        movlw   hrs
        movwf   fsr             ; address of the hrs register to fsr
        btfsc   stat1, hm       ; display hrs and min? (bit=0?)
        incf    fsr, f          ; no  => in fsr there is the min address
                                ; yes => nothing happensnic se nedje
        bcf     status, carry   ; clear carry for rotation
        rrf     cnt_m, w        ; multiplex counter/2 is in w
                                ; cnt_m is odd  => carry = 1
                                ; cnt_m is even => carry = 0
        iorlw   00h             ; sets the zero bit in the status
        btfss   status, zero    ; original value cnt_m = 0 or 1? (Z=1?)
        incf    fsr, f          ; no  => fsr points to next register:
                                ;        sec, in the min, sec mode
                                ;        min, in the hrs, min mode
                                ; yes => nothing happens
        movf    indf, w         ; selected register to w
        movwf   digit           ; and w to digit
        btfss   status, carry   ; cnt_m odd? (C=1?)
        swapf   digit, f        ; no  => swap nibbles
                                ; yes => without changes
        movlw   b'00001111'
        andwf   digit, w        ; mask. One BCD digit is a result
        call    tab1            ; BCD digit to segments
        movwf   portb           ; segments codes to port B
        movf    cnt_m, w        ; cnt_m to w
        call    tab2            ; decode one anode according to the cnt_m
        movwf   porta           ; activate the appropriate anode
        bcf     status, carry   ; clear carry for rotation
        rrf     sec, w          ; seconds/2 to w
        btfsc   status, carry   ; even second? (C=0?)
        goto    disp_a          ; no  => point off
                                ; yes => set the DP2 diode
        movf    cnt_m, w
        xorlw   01h
        btfsc   status, zero    ; cnt_m = 1? (Z=1?)
        bcf     portb, dp       ; yes => diodu DP2 on
                                ; no  => nothing happens
disp_a  incf    cnt_m, f        ; increment cnt_m
        movlw   b'00000011'
        andwf   cnt_m, f        ; mask. Now cnt_m counts to 3 only
        retlw   00h             ; end
;***************************************************************************
; The routine provides debug of pushbuttons and setting of appropriate bits
; of the stat1 register for controlling of the sate of the system. The
; routine reads buttons every 20ms (using the cnt_m register) and in case
; of the button is pressed at least twice, it is evaluated as a pressed one.
button  movf    cnt_m, w
        btfss   status, zero    ; cnt_m = 0? (Z=1?)
        retlw   00h             ; no  => end
        movlw   b'11111111'     ; yes => pushbuttons read
        movwf   porta           ; disconect display anodes
        clrf    portb           ; bit 0 of the portu B is cleared (common
        movlw   b'11111110'     ; pin for pushbuttons)
        tris    portb           ; RB0 output, others inputs
        movlw   d'56'
        movwf   cnt_d           ; delay counter = 56
push_a  decfsz  cnt_d, f        ; delay for bus stabilization
        goto    push_a          ; about 200us
        movf    portb, w        ; port B reading (buttons)
                                ; button pressed => log 0
        xorlw   b'11111111'     ; complement
        andlw   b'00111110'     ; mask unused bits
                                ; button pressed => log 1
        movwf   new             ; update to the new register
        clrf    portb           ; port B clear before changing the direction
        movlw   00h
        tris    portb           ; port B is an output again
        btfss   stat1, push     ; button still pressed? (bit push=1?)
        goto    push_d          ; no  => continue processing
                                ; yes => button release test
        clrf    old             ; clear old
        movf    new, w
        btfss   status, zero    ; new = 0? (Z=1?)
        goto    push_b          ; no  => autorepeat test
        bcf     stat1, push     ; yes => button released, bit cleared
        bcf     stat1, ah       ; neither autorepeat of hours
        bcf     stat1, am       ; nor minutes
        retlw   00h             ; end
push_b  decfsz  cnt_a, f        ; autorepeat counter overflew?
        retlw   00h             ; no  => end
        movlw   .12             ; yes => set repeat delay
        movwf   cnt_a           ; to 12*20ms = 240ms
        btfss   stat1, ah       ; autorepeat of hours? (bit ah=1?)
        goto    push_c          ; no  => autorepeat of minutes test
        movlw   hrs             ; yes => increment hours
        movwf   fsr
        movlw   24h
        movwf   maxim           ; hours selection
        call    update          ; hours update
        retlw   00h             ; and end
push_c  btfss   stat1, am       ; autorepeat of minutes? (bit am=1?)
        retlw   00h             ; no  => end
        movlw   min             ; increment fo minutes
        movwf   fsr
        movlw   60h
        movwf   maxim           ; selection of minutes
        call    update          ; minutes update
        retlw   00h             ; and end
push_d  movf    old, w          ; old state of buttons to w
        andwf   new, w          ; w contains new and old
        btfss   status, zero    ; new and old = 0? (Z=1?)
        goto    push_e          ; no  => button pressed
        movf    new, w          ; yes => copy new state to the old one
        movwf   old
        retlw   00h             ; and end
push_e  bsf     stat1, push     ; button pressed, set the bit
        movlw   d'50'
        movwf   cnt_a           ; delay before autorepeat 50*20ms = 1000ms
        btfss   new, 1          ; pressed the start/stop button? (bit1=1?)
        goto    push_f          ; no  => continue scanning
        movlw   b'00000010'     ; yes => complement of the stop bit, others unchanged
        xorwf   stat1, f
        btfsc   stat1, stop     ; clock stpooed? (bit stop=1?)
        clrf    sec             ; yes => clear seconds and end
        retlw   00h             ; no  => end only
push_f  btfss   new, 2          ; the switch button pressed? (bit2=1?)
        goto    push_g          ; no  => continue scanning
        movlw   b'00000100'     ; yes => complement of the hm bit, others unchanged
        xorwf   stat1, f
        retlw   00h             ; and end
push_g  btfss   new, 3          ; the set hrs button pressed? (bit3=1?)
        goto    push_h          ; no  => continue scanning
        movlw   hrs             ; yes => increment hours
        movwf   fsr
        movlw   24h
        movwf   maxim           ; selection of hours
        call    update          ; update hours
        bsf     stat1, ah       ; there will be the autorepeat of hours
        retlw   00h             ; and end
push_h  btfss   new, 4          ; the set min button pressed? (bit4=1?)
        goto    push_i          ; no  => test procedure
        movlw   min             ; yes => increment of minutes
        movwf   fsr
        movlw   60h
        movwf   maxim           ; selection of minutes
        call    update          ; update minutes
        bsf     stat1, am       ; there will be the autorepeat of minutes
        retlw   00h             ; and end
push_i  bsf     stat1, test     ; test mode
        retlw   00h             ; end
;***************************************************************************
; The routine debugs pushbuttons in test mode. The buttons are scanned every
; 20ms (using the cnt_m register) and if the button is pressed at least twice
; the button is evaluated as a pressed one.
pbtest  movf    cnt_m, w
        btfss   status, zero      ; cnt_m = 0? (Z=1?)
        retlw   00h             ; no  => end
                                ; yes => scan the buttons
        movlw   b'11111111'
        movwf   porta           ; disconnect display anodes
        clrf    portb           ; clear bit 0 of port B (common
        movlw   b'11111110'     ; pin of buttons)
        tris    portb           ; RB0 output, others inputs
        movlw   .56
        movwf   cnt_d           ; delay counter = 56
pbte_a  decfsz  cnt_d, f        ; delay for bus stabilization
        goto    pbte_a          ; about 200us
        movf    portb, w        ; read port B (pushbuttons)
                                ; button pressed => log 0
        xorlw   b'11111111'     ; complement
        andlw   b'11000000'     ; mask unused bits
                                ; button pressed => log 1
        movwf   new             ; copy to the new register
        clrf    portb           ; port B clear before changing the direction
        movlw   00h
        tris    portb           ; port B is an output again
        btfss   stat1, push     ; button still pressed? (bit push=1?)
        goto    pbte_c          ; no  => continue processing
                                ; yes => button release test
        clrf    old             ; clear old
        movf    new, w
        btfss   status, zero    ; new = 0? (Z=1?)
        goto    pbte_b          ; no  => test autorepeat
        bcf     stat1, push     ; yes => button pressed, bit cleared
        bcf     stat1, ad       ; no more autorepeat of LEDs and segments
        retlw   00h             ; end
pbte_b  decfsz  cnt_a, f        ; autorepeat counter overflew?
        retlw   00h             ; no  => end
        movlw   .10             ; yes => set the autorepeat delay
        movwf   cnt_a           ; to 10*20ms = 200ms
        btfss   stat1, ad       ; autorepeat of LEDs and segments? (bit ad=1?)
        retlw   00h             ; no  => end
        movlw   0ffh            ; yes => next LED or segment on
        xorwf   segmt, w
        bcf     status, carry   ; clear carry for rotation
        btfss   status, zero    ; segmt = 0ffh (Z=1?)
        bsf     status, carry   ; no  => carry = 1
                                ; yes => carry = 0 (default)
        btfss   status, zero    ; segmt = 0ffh (Z=1?)
        bsf     stat1, chang    ; no  => change enabled
                                ; yes => bit unchanged
        rlf     segmt, f        ; select next segmt or segment
        retlw   00h             ; and end
pbte_c  movf    old, w          ; old state of buttons to w
        andwf   new, w          ; w contains new and old
        btfss   status, zero    ; new and old = 0? (Z=1?)
        goto    pbte_d          ; no  => button pressed
        movf    new, w          ; yes => copy new state to the old one
        movwf   old
        retlw   00h             ; and end
pbte_d  bsf     stat1, push     ; button pressed, set the bit
        movlw   .50
        movwf   cnt_a           ; delay before autorepeat 50*20ms = 1000ms
        btfss   new, 6          ; the switch button pressed? (bit6=1?)
        goto    pbte_e          ; no  => the test button pressed
        movlw   0ffh            ; yes => next LED or segment on
        xorwf   segmt, w
        bcf     status, carry   ; clear carry for rotation
        btfss   status, zero    ; segmt = 0ffh (Z=1?)
        bsf     status, carry   ; no  => carry = 1
                                ; yes => carry = 0 (default)
        btfss   status, zero    ; segmt = 0ffh (Z=1?)
        bsf     stat1, chang    ; no  => change enabled
                                ; yes => bit unchanged
        rlf     segmt, f        ; select next segmt or segment
        bsf     stat1, ad       ; there will be an autorepeat of LEDs and segments
        retlw   00h             ; and end
pbte_e  movlw   b'11111111'     ; set port A
        movwf   porta           ; drivers disabled => nothing lights
        clrf    portb           ; clear port B
        bcf     stat1, test     ; real time mode
        bcf     stat1, push     ; button released, bit cleared
        bcf     stat1, ad       ; no more autorepeat of LEDs and segments
        bcf     stat1, chang    ; change disabled (for reliability only)
        clrf    cnt_a           ; clear autorepeat counter
        retlw   00h             ; end
; END OF SUB.INC ==========================================================
